@@ -1,4 +1,4 @@
-# Time-stamp: <2012-09-14 02:32:11 Tao Liu>
+# Time-stamp: <2013-03-05 16:53:49 Tao Liu>
 
 """Module Description
 
@@ -21,6 +21,7 @@ import sys
 import os
 import re
 import logging
+from argparse import ArgumentError
 from subprocess import Popen, PIPE
 from math import log
 from MACS2.IO.cParser import BEDParser, ELANDResultParser, ELANDMultiParser, \
@@ -90,8 +91,17 @@ def opt_validate ( options ):
             sys.exit(1)
 
     # shiftsize>0
+    if options.shiftsize:               # only if --shiftsize is set, it's true
+        options.extsize = 2 * options.shiftsize
+    else:                               # if --shiftsize is not set
+        options.shiftsize = options.extsize / 2
     if options.shiftsize <= 0 :
-        logging.error("--shiftsize must > 0!")
+        logging.error("--extsize must > 1 and --shiftsize must > 0!")
+        sys.exit(1)
+
+    # refine_peaks, call_summits can't be combined with --broad
+    if options.broad and (options.refine_peaks or options.call_summits):
+        logging.error("--broad can't be combined with --refine-peaks or --call-summits!")
         sys.exit(1)
 
     if options.pvalue:
@@ -176,6 +186,11 @@ def opt_validate ( options ):
     else:
         options.argtxt += "# Broad region calling is off\n"
 
+    if options.refine_peaks:
+        options.argtxt += "# Refining peak for read balance is on\n"
+    if options.call_summits:
+        options.argtxt += "# Searching for subpeak summits is on\n"
+
     if options.halfext:
         options.argtxt += "# MACS will make 1/2d size fragments\n"
 
@@ -184,9 +199,112 @@ def opt_validate ( options ):
 
     return options
 
+def diff_opt_validate ( options ):
+    """Validate options from a OptParser object.
+
+    Ret: Validated options object.
+    """
+    # format
+    options.gzip_flag = False           # if the input is gzip file
+    
+#    options.format = options.format.upper()
+    # fox this stuff
+#    if True: pass
+#    elif options.format == "AUTO":
+#        options.parser = guess_parser
+#    else:
+#        logging.error("Format \"%s\" cannot be recognized!" % (options.format))
+#        sys.exit(1)
+    
+    if options.peaks_pvalue:
+        # if set, ignore qvalue cutoff
+        options.peaks_log_qvalue = None
+        options.peaks_log_pvalue = log(options.peaks_pvalue,10)*-1
+        options.track_score_method = 'p'
+    else:
+        options.peaks_log_qvalue = log(options.peaks_qvalue,10)*-1
+        options.peaks_log_pvalue = None
+        options.track_score_method = 'q'
+
+    if options.diff_pvalue:
+        # if set, ignore qvalue cutoff
+        options.log_qvalue = None
+        options.log_pvalue = log(options.diff_pvalue,10)*-1
+        options.score_method = 'p'
+    else:
+        options.log_qvalue = log(options.diff_qvalue,10)*-1
+        options.log_pvalue = None
+        options.score_method = 'q'
+    
+    # output filenames
+    options.peakxls = options.name+"_diffpeaks.xls"
+    options.peakbed = options.name+"_diffpeaks.bed"
+    options.peak1xls = options.name+"_diffpeaks_by_peaks1.xls"
+    options.peak2xls = options.name+"_diffpeaks_by_peaks2.xls"
+    options.bdglogLR = options.name+"_logLR.bdg"
+    options.bdgpvalue = options.name+"_logLR.bdg"
+    options.bdglogFC = options.name+"_logLR.bdg"
+    
+    options.call_peaks = True
+    if not (options.peaks1 == '' or options.peaks2 == ''):
+        if options.peaks1 == '':
+            raise ArgumentError('peaks1', 'Must specify both peaks1 and peaks2, or neither (to call peaks again)')
+        elif options.peaks2 == '':
+            raise ArgumentError('peaks2', 'Must specify both peaks1 and peaks2, or neither (to call peaks again)')
+        options.call_peaks = False
+        options.argtxt = "\n".join((
+            "# ARGUMENTS LIST:",\
+            "# name = %s" % (options.name),\
+#            "# format = %s" % (options.format),\
+            "# ChIP-seq file 1 = %s" % (options.t1bdg),\
+            "# control file 1 = %s" % (options.c1bdg),\
+            "# ChIP-seq file 2 = %s" % (options.t2bdg),\
+            "# control file 2 = %s" % (options.c2bdg),\
+            "# Peaks, condition 1 = %s" % (options.peaks1),\
+            "# Peaks, condition 2 = %s" % (options.peaks2),\
+            ""
+            ))
+    else:
+        options.argtxt = "\n".join((
+            "# ARGUMENTS LIST:",\
+            "# name = %s" % (options.name),\
+#            "# format = %s" % (options.format),\
+            "# ChIP-seq file 1 = %s" % (options.t1bdg),\
+            "# control file 1 = %s" % (options.c1bdg),\
+            "# ChIP-seq file 2 = %s" % (options.t2bdg),\
+            "# control file 2 = %s" % (options.c2bdg),\
+            ""
+            ))
+         
+        if options.peaks_pvalue:
+            options.argtxt +=  "# treat/control -log10(pvalue) cutoff = %.2e\n" % (options.peaks_log_pvalue)
+            options.argtxt +=  "# treat/control -log10(qvalue) will not be calculated and reported as -1 in the final output.\n"
+        else:
+            options.argtxt +=  "# treat/control -log10(qvalue) cutoff = %.2e\n" % (options.peaks_log_qvalue)
+        
+    if options.diff_pvalue:
+        options.argtxt +=  "# differential pvalue cutoff = %.2e\n" % (options.log_pvalue)
+        options.argtxt +=  "# differential qvalue will not be calculated and reported as -1 in the final output.\n"
+    else:
+        options.argtxt +=  "# differential qvalue cutoff = %.2e\n" % (options.log_qvalue)
+
+    # logging object
+    logging.basicConfig(level=(4-options.verbose)*10,
+                        format='%(levelname)-5s @ %(asctime)s: %(message)s ',
+                        datefmt='%a, %d %b %Y %H:%M:%S',
+                        stream=sys.stderr,
+                        filemode="w"
+                        )
+    
+    options.error   = logging.critical        # function alias
+    options.warn    = logging.warning
+    options.debug   = logging.debug
+    options.info    = logging.info
+
+    return options
 
 def opt_validate_diff ( optparser ):
-    """Validate options from a OptParser object.
+    """Validate options from a OptParser object. (temporarily not used at all)
 
     This parser is for macsdiffrun.
 
@@ -513,3 +631,131 @@ def opt_validate_refinepeak ( options ):
     options.info    = logging.info
 
     return options
+
+def opt_validate_predictd ( options ):
+    """Validate options from a OptParser object.
+
+    Ret: Validated options object.
+    """
+    # gsize
+    try:
+        options.gsize = efgsize[options.gsize]
+    except:
+        try:
+            options.gsize = float(options.gsize)
+        except:
+            logging.error("Error when interpreting --gsize option: %s" % options.gsize)
+            logging.error("Available shortcuts of effective genome sizes are %s" % ",".join(efgsize.keys()))
+            sys.exit(1)
+
+    # format
+    options.gzip_flag = False           # if the input is gzip file
+    
+    options.format = options.format.upper()
+    if options.format == "ELAND":
+        options.parser = ELANDResultParser
+    elif options.format == "BED":
+        options.parser = BEDParser
+    elif options.format == "ELANDMULTI":
+        options.parser = ELANDMultiParser
+    elif options.format == "ELANDEXPORT":
+        options.parser = ELANDExportParser
+    elif options.format == "SAM":
+        options.parser = SAMParser
+    elif options.format == "BAM":
+        options.parser = BAMParser
+        options.gzip_flag = True
+    elif options.format == "BAMPE":
+        options.parser = BAMPEParser
+        options.gzip_flag = True
+        options.nomodel = True
+    elif options.format == "BOWTIE":
+        options.parser = BowtieParser
+    elif options.format == "AUTO":
+        options.parser = guess_parser
+    else:
+        logging.error("Format \"%s\" cannot be recognized!" % (options.format))
+        sys.exit(1)
+    
+    # uppercase the format string 
+    options.format = options.format.upper()
+
+    # upper and lower mfold
+    options.lmfold = options.mfold[0]
+    options.umfold = options.mfold[1]
+    if options.lmfold > options.umfold:
+        logging.error("Upper limit of mfold should be greater than lower limit!" % options.mfold)
+        sys.exit(1)
+    
+    options.modelR  = options.rfile+"_model.R"
+
+    # logging object
+    logging.basicConfig(level=(4-options.verbose)*10,
+                        format='%(levelname)-5s @ %(asctime)s: %(message)s ',
+                        datefmt='%a, %d %b %Y %H:%M:%S',
+                        stream=sys.stderr,
+                        filemode="w"
+                        )
+    
+    options.error   = logging.critical        # function alias
+    options.warn    = logging.warning
+    options.debug   = logging.debug
+    options.info    = logging.info
+
+    return options
+
+
+def opt_validate_pileup ( options ):
+    """Validate options from a OptParser object.
+
+    Ret: Validated options object.
+    """
+    # format
+
+    options.gzip_flag = False           # if the input is gzip file
+    
+    options.format = options.format.upper()
+    if options.format == "ELAND":
+        options.parser = ELANDResultParser
+    elif options.format == "BED":
+        options.parser = BEDParser
+    elif options.format == "ELANDMULTI":
+        options.parser = ELANDMultiParser
+    elif options.format == "ELANDEXPORT":
+        options.parser = ELANDExportParser
+    elif options.format == "SAM":
+        options.parser = SAMParser
+    elif options.format == "BAM":
+        options.parser = BAMParser
+        options.gzip_flag = True
+    elif options.format == "BOWTIE":
+        options.parser = BowtieParser
+    elif options.format == "AUTO":
+        options.parser = guess_parser
+    else:
+        logging.error("Format \"%s\" cannot be recognized!" % (options.format))
+        sys.exit(1)
+    
+    # uppercase the format string 
+    options.format = options.format.upper()
+
+    # logging object
+    logging.basicConfig(level=(4-options.verbose)*10,
+                        format='%(levelname)-5s @ %(asctime)s: %(message)s ',
+                        datefmt='%a, %d %b %Y %H:%M:%S',
+                        stream=sys.stderr,
+                        filemode="w"
+                        )
+    
+    options.error   = logging.critical        # function alias
+    options.warn    = logging.warning
+    options.debug   = logging.debug
+    options.info    = logging.info
+
+    # extsize
+    if options.extsize <= 0 :
+        logging.error("--extsize must > 0!")
+        sys.exit(1)
+
+    return options
+
