@@ -1,5 +1,5 @@
 # cython: profile=True
-# Time-stamp: <2012-08-02 11:16:47 Tao Liu>
+# Time-stamp: <2013-03-05 16:49:06 Tao Liu>
 
 """Description: MACS 2 main executable
 
@@ -162,7 +162,7 @@ def run( args ):
             options.d = options.tsize
         else:
             options.d=options.shiftsize*2
-        info("#2 Use %d as shiftsize, %d as fragment length" % (options.shiftsize,options.d))
+        info("#2 Use %d as fragment length" % (options.d))
         options.scanwindow=2*options.d  # remove the effect of --bw
     else:
         try:
@@ -247,6 +247,19 @@ def run( args ):
                             opt = options
                             )
     peakdetect.call_peaks()
+
+    #call refinepeak if needed.
+    if options.refine_peaks:
+        info("#3 now put back duplicate reads...")
+        treat.addback_dups()
+        info("#3 calculate reads balance to refine peak summits...")
+        refined_peaks = treat.refine_peak_from_tags_distribution ( peakdetect.peaks, options.d, 0 )
+        info("#3 reassign scores for newly refined peak summits...")
+        peakdetect.peaks = peakdetect.scoretrack.reassign_peaks( refined_peaks ) # replace
+        #info("#3 write to file: %s ..." % options.name+"_refined_peaks.encodePeak" )
+        #refinedpeakfile = open(options.name+"_refined_peaks.encodePeak", "w")
+        #refined_peaks.write_to_narrowPeak (refinedpeakfile, name_prefix="%s_refined_peak_", name=options.name, score_column=score_column, trackline=options.trackline )
+
     #diag_result = peakdetect.diag_result()
     #4 output
     #4.1 peaks in XLS
@@ -296,6 +309,10 @@ def run( args ):
                                           score_column=score_column, trackline=options.trackline )
     ofhd_summits.close()
 
+
+
+    info("Done!")
+    
 def cal_max_dup_tags ( genome_size, tags_number, p=1e-5 ):
     """Calculate the maximum duplicated tag number based on genome
     size, total tag number and a p-value based on binomial
